/*
 * main.c
 *
 *  Created on: Nov 10, 2011
 *      Author: lkerofsky
 */
// code to test all combinations of level/QP and verify that structured level limits restrict
// dequantized coeffcients to 16-bits i.e. -32768 to 32767
#include <stdio.h>
#define TIGHT_LEVEL_LIMITS	// tight level limits derived from 32x32 transform and QP=0-5
#define N_BLOCKSIZE (4) // 4,8,16,32
#define MIN_BLOCKSIZE (2)
#define MAX_BLOCK_SIZE (MIN_BLOCKSIZE+N_BLOCKSIZE-1)
#define MIN(x, y) ((x)<(y) ? (x):(y))
#define MAX(x, y) ((x)>(y) ? (x):(y))
int g[6]  =   {40,45,51,57,64,72};					// dequant factors
#ifdef TIGHT_LEVEL_LIMITS
int LB[6] = {13106,11650,10279,9197,8191,7281} ;	// base level limits
#else
int LB[6] = {819,728,642,574,511,455};	// base level limits
#endif
// verify structured level limits imply dequantized coefficient limits
int main(int argc, char **argv)
{
	int AbsoluteLevel;
	int Sign;
	int M;
	int MaxAbsoluteLevel;
	int LevelBound;
	int level,offset;
	int coeffQ;
	int QP;
	int MaxQP;
	int IQ;
	int MaxCoeff[N_BLOCKSIZE],MinCoeff[N_BLOCKSIZE];
	int AllMin,AllMax;
	int B;

	MaxAbsoluteLevel = (1<<15)-1;
	MaxQP = 51;
	AllMin = 0;
	AllMax = 0;
	for(M=MIN_BLOCKSIZE;M<=MAX_BLOCK_SIZE;M++)
	{
		MaxCoeff[M-MIN_BLOCKSIZE] = 0;
		MinCoeff[M-MIN_BLOCKSIZE] = 0;
	}
	for(B=8;B<=10;B++)							// loop over bitdepths 8 and 10
	for(QP=0;QP<=MaxQP;QP++)					// loop over QP 0 to 51
	{
		IQ 	= g[QP%6];
		for(M=MIN_BLOCKSIZE;M<=MAX_BLOCK_SIZE;M++)	// loop over block size M=2 to 5
		{
			offset = 1<<(M-2+B-8);
#ifdef TIGHT_LEVEL_LIMITS
			LevelBound=(LB[QP%6]<<(B-8))>>(5-M+QP/6);
#else
			LevelBound=(LB[QP%6]<<(M-1+B-8))>>(QP/6);
#endif
			// loop over level absolute value and sign
			for(AbsoluteLevel=1;AbsoluteLevel<=MaxAbsoluteLevel;AbsoluteLevel++)
			for(Sign=-1;Sign<=1;Sign+=2)
			{

				level = MIN(AbsoluteLevel,LevelBound);
				level = Sign*level;

				coeffQ 	=  ((level*IQ << (QP/6)) + offset)>>(M-1+B-8);
				if(coeffQ>MaxCoeff[M-MIN_BLOCKSIZE])
					MaxCoeff[M-MIN_BLOCKSIZE] = coeffQ;
				if(coeffQ<MinCoeff[M-MIN_BLOCKSIZE])
					MinCoeff[M-MIN_BLOCKSIZE] = coeffQ;
				AllMin = MIN(AllMin,MinCoeff[M-MIN_BLOCKSIZE]);
				AllMax = MAX(AllMax,MaxCoeff[M-MIN_BLOCKSIZE]);
			}
		}
	}
	printf("%1s\t%7s\t%7s\n","M","Min","Max");
	for(M=MIN_BLOCKSIZE;M<=MAX_BLOCK_SIZE;M++)
	{
		printf("%2d\t%7d\t%7d\n",M,MinCoeff[M-MIN_BLOCKSIZE],MaxCoeff[M-MIN_BLOCKSIZE]);
	}

	printf("\n\nAll coefficients in range %d to %d\n",AllMin,AllMax);

	fflush(stdout);
	return 1;
}
